==> Zelig_2.3-2.tar.gz: Zelig/R/MCMC/qi.MCMCZelig.R <==

qi.MCMCZelig <- function(object, simpar=NULL, x, x1 = NULL, y = NULL, ...) {

  model <- object$zelig
  qi <- list()
  check <- FALSE
  if (model %in% c("MCMClogit", "MCMCprobit", "MCMCoprobit", "MCMCmnl")) 
    check <- TRUE
  
  if (model %in% c("MCMClogit","MCMCprobit", "MCMCregress",
                   "MCMCpoisson","MCMCtobit")) {

    if (model == "MCMClogit") {
      coef <- object$coefficients
      eta <- coef %*% t(x)
      pr <- ev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(pr) <- dimnames(ev) <- dimnames(eta)
      ev <- 1/(1+exp(-eta))
      for (i in 1:ncol(ev)) 
        pr[,i] <- as.character(rbinom(length(ev[,i]), 1, ev[,i])) 
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")
    }
    else if (model == "MCMCprobit") {
      coef <- object$coefficients
      eta <- coef %*% t(x)
      pr <- ev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(pr) <- dimnames(ev) <- dimnames(eta)
      ev <- pnorm(eta)
      for (i in 1:ncol(ev)) 
        pr[,i] <- as.character(rbinom(length(ev[,i]), 1, ev[,i]))
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")
    }
    else if (model =="MCMCregress") {
      coef <- object$coefficients[,1:(ncol(object$coefficients)-1)]
      eta <- coef %*% t(x)
      ev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(ev) <- dimnames(eta)
      ev <- eta
      qi$ev <- ev
      qi.name <- list(ev = "Expected Values: E(Y|X)")
    }
    else if (model =="MCMCtobit") {
      coef <- object$coefficients[,1:(ncol(object$coefficients)-1)]
      sig2 <- object$coefficients[,ncol(object$coefficients)]
      sig <- sqrt(sig2)
      eta <- coef %*% t(x)
      ev <- cev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(cev) <- dimnames(ev) <- dimnames(eta)
      
      L2 <- (object$above-eta)/sig
      L1 <- (object$below-eta)/sig
      #cev <- eta + sig*(dnorm(L1)-dnorm(L2))/(pnorm(L2)-pnorm(L1))
      if (object$below==-Inf) temp1<-0
        else temp1 <- pnorm(L1)*object$below
      if (object$above==Inf) temp2<-0
        else temp2 <- (1-pnorm(L2))*object$above

      ev <- temp1+eta*(pnorm(L2)-pnorm(L1))+sig*(dnorm(L1)-dnorm(L2))+temp2

    qi$ev <- ev

    qi.name <- list(ev = "Expected Values: E(Y|X)")
    }
    else if (model == "MCMCpoisson") {
      coef <- object$coefficients
      eta <- coef %*% t(x)
      pr <- ev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(pr) <- dimnames(ev) <- dimnames(eta)
      ev <- exp(eta)
      for (i in 1:ncol(ev)) 
        pr[,i] <- rpois(length(ev[,i]), ev[,i])
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")

    }
    
        
    if (!is.null(x1)) {
        eta1 <- coef %*% t(x1)
      if (model == "MCMClogit") {
        ev1 <- 1/(1+exp(-eta1))
        rr <-ev1/ev
        fd <-ev1-ev

        qi$fd <- fd
        qi$rr <- rr

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

        qi.name$rr <- "Risk Ratios: P(Y=1|X1)/P(Y=1|X)"
      }
      else if (model == "MCMCprobit") {
        ev1 <- pnorm(eta1)
                rr <-ev1/ev
        fd <-ev1-ev

        qi$fd <- fd
        qi$rr <- rr

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

        qi.name$rr <- "Risk Ratios: P(Y=1|X1)/P(Y=1|X)"
      }
      else if (model == "MCMCregress") {
        ev1 <- eta1
        fd <-ev1-ev

        qi$fd <- fd

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"
      }
      else if (model == "MCMCtobit") {
        L2 <- (object$above-eta1)/sig
        L1 <- (object$below-eta1)/sig
        #cev <- eta + sig*(dnorm(L1)-dnorm(L2))/(pnorm(L2)-pnorm(L1))

        if (object$below==-Inf) temp1<-0
        else temp1 <- pnorm(L1)*object$below
        if (object$above==Inf) temp2<-0
        else temp2 <- (1-pnorm(L2))*object$above

        ev1 <- temp1+eta*(pnorm(L2)-pnorm(L1))+sig*(dnorm(L1)-dnorm(L2))+temp2
        
        fd <-ev1-ev

        qi$fd <- fd

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"
      }        
      else if (model == "MCMCpoisson") {
        ev1 <- exp(eta1)
        fd <-ev1-ev

        qi$fd <- fd

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

      }
   }

    if (!is.null(y)) {
    yvar <- matrix(rep(y, nrow(simpar)), nrow = nrow(simpar), byrow = TRUE)
    tmp.ev <- yvar - qi$ev
    if (check) 
      tmp.pr <- yvar - as.integer(qi$pr)
   else
      tmp.pr <- yvar - qi$pr
    qi$ate.ev <- matrix(apply(tmp.ev, 1, mean), nrow = nrow(simpar))
    qi.name$ate.ev <- "Average Treatment Effect: Y - EV"
    if (model %in% c("MCMClogit", "MCMCprobit", "MCMCpoisson"))
      {
        qi$ate.pr <- matrix(apply(tmp.pr, 1, mean), nrow = nrow(simpar))
        qi.name$ate.pr <- "Average Treatment Effect: Y - PR"
      }
  }

    list(qi=qi, qi.name=qi.name)    
  }
  else if ((model =="MCMCoprobit") || (model == "MCMCmnl")) {
    if (model == "MCMCoprobit") {
      library(stats)
      p <- dim(model.matrix(object, data=eval(object$data)))[2]
      coef <- object$coefficients[,1:p]
      eta <- coef %*% t(x)
      
      level <- ncol(object$coefficients)-p+2
      gamma<-matrix(NA, nrow(object$coefficients), level+1) 

      gamma[,1] <- rep(-Inf, nrow(gamma))
      gamma[,2] <- rep(0, nrow(gamma))
      gamma[,ncol(gamma)]<-rep(Inf, nrow(gamma))

      if (ncol(gamma)>3)
        gamma[,3:(ncol(gamma)-1)] <- object$coefficients[,(p+1):ncol(object$coefficients)]
 

      ev <- array(NA, c(nrow(eta), level, ncol(eta)))
      pr <- matrix(NA, nrow(eta), ncol(eta))
#      dimnames(pr)[1] <- dimnames(ev)[1] <- dimnames(eta)[1]
#      dimnames(pr)[2] <- dimnames(ev)[3] <- dimnames(eta)[2]

      for (j in 1:level)
        ev[,j,] <- pnorm(gamma[,j+1]-eta) - pnorm(gamma[,j]-eta)

      for (j in 1:nrow(pr)) {
       mu <- eta[j,]
       pr[j,]<-as.character(cut(mu, gamma[j,], labels=as.factor(1:level)))   
     }
       

      #        t(t(1:nrow(pr))%*%pr)
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")      
    }
    else if (model == "MCMCmnl") {
      library(stats)
      resp <- model.response(model.frame(object))
      level <- length(table(resp))

      p <- dim(model.matrix(eval(object),data=eval(object$data)))[2]
      
      coef <- object$coefficients


      eta <- array(NA, c(nrow(coef),level, nrow(x)))
      
      eta[,1,]<-matrix(0, dim(eta)[1],dim(eta)[3])
      for (j in 2:level) {
        ind <- (1:p)*(level-1)-(level-j)
        eta[,j,]<- coef[,ind]%*%t(x)
      }

      eta<-exp(eta)
                                      
      ev <- array(NA, c(nrow(coef), level, nrow(x)))
      pr <- matrix(NA, nrow(coef), nrow(x))
      #dimnames(pr)[1] <- dimnames(ev)[1] <- dimnames(eta)[1]

      for (k in 1:nrow(x))
       for (j in 1:level)
         {
           ev[,j,k] <- eta[,j,k]/rowSums(eta[,,k])
         }

      for (k in 1:nrow(x)) {             
        probs <- as.matrix(ev[,,k])
        temp <- apply(probs, 1, FUN=rmultinom, n=1, size=1)
        temp <- as.matrix(t(temp)%*%(1:nrow(temp)))
        pr <- apply(temp,2,as.character)
    }
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")      
    }

    
    if (!is.null(x1)) {

      if (model == "MCMCoprobit") {
 
        eta1 <- coef %*% t(x1)
      
      ev1 <- array(NA, c(nrow(eta), level, ncol(eta)))
      
      for (j in 1:level)
        ev1[,j,] <- pnorm(gamma[,j+1]-eta1) - pnorm(gamma[,j]-eta1)

        rr <-ev1/ev
        fd <-ev1-ev

        qi$fd <- fd
        qi$rr <- rr

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

        qi.name$rr <- "Risk Ratios: P(Y=1|X1)/P(Y=1|X)"
        
      }
      else if (model == "MCMCmnl") {

      eta1 <- array(NA, c(nrow(coef),level, nrow(x1)))
      
      eta1[,1,]<-matrix(0, dim(eta1)[1],dim(eta1)[3])
      for (j in 2:level) {
        ind <- (1:p)*(level-1)-(level-j)
        eta1[,j,]<- coef[,ind]%*%t(x1)
      }

      eta1<-exp(eta1)
                                       
      ev1 <- array(NA, c(nrow(eta1), level, nrow(x1)))

      for (k in 1:nrow(x))
       for (j in 1:level)
         {
           ev1[,j,k] <- eta1[,j,k]/rowSums(eta1[,,k])
         }

              rr <-ev1/ev
        fd <-ev1-ev

        qi$fd <- fd
        qi$rr <- rr

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

        qi.name$rr <- "Risk Ratios: P(Y=1|X1)/P(Y=1|X)"
      
    }
    }

    if (!is.null(y)) {
    yvar <- matrix(rep(y, nrow(simpar)), nrow = nrow(simpar), byrow =
    TRUE)

    levels.names<-levels(as.factor(y))
    yvar1<- pr1 <-array(NA, c(nrow(yvar), level, ncol(yvar)))


    for (j in 1:nrow(yvar))
      {
        yvar1[j,,]<-t(class.ind(yvar[j,], levels.names))
        if (check)
         pr1[j,,]<-t(class.ind(as.integer(qi$pr[j,]), levels.names))
        else
          pr1[j,,]<-t(class.ind(qi$pr[j,],levels.names))
      }
                        
    tmp.ev <- yvar1 - qi$ev
    tmp.pr <- yvar1 - pr1
    qi$ate.ev <- matrix(apply(tmp.ev, 2, rowMeans), nrow = nrow(simpar))
    qi.name$ate.ev <- "Average Treatment Effect: Y - EV"
    if (model %in% c("MCMCoprobit", "MCMCmnl"))
      {
        qi$ate.pr <- matrix(apply(tmp.pr, 2, rowMeans), nrow = nrow(simpar))
        qi.name$ate.pr <- "Average Treatment Effect: Y - PR"
      }
  }

    list(qi=qi, qi.name=qi.name)    


    
  }
  else if (model == "MCMChierEI" || model == "MCMCdynamicEI") {
    if (!any(class(x)=="cond")) stop("set 'cond=TRUE' in setx.\n")
    else
      {
        coef <- object$coefficients
        n <- nrow(x)
        if (is.null(object$N))
          N<-rep(1,n)
        else N <- eval(object$N)
        ev <- array(NA, c(nrow = nrow(coef), 2,2, n))
        pr <- array(NA, c(nrow = nrow(coef), 2,2, n))
        nlen<-length(coef[,1])
        for (j in 1:2) {
          ev[,j,1,] <- t(apply(coef[,((1:n)+(j-1)*n)],
                               1,"*",  x[,j])*N)
          ev[,j,2,] <- t(apply((1-coef[,((1:n)+(j-1)*n)]), 1,"*",
                               x[,j])*N)
          for (i in 1:n)
            {
              size<-round(x[i,j]*N[i])
              pr[,j,1,i] <-rbinom(prob=coef[,(i+(j-1)*n)],  n=nlen, size=size)

              pr[,j,2,i] <- x[i,j]*N[i]-pr[,j,1,i]
            }
          
        }
        
        dimnames(ev)[[4]] <- dimnames(pr)[[4]] <- rownames(x)
        dimnames(ev)[[2]] <- dimnames(pr)[[2]] <- colnames(x)
        dimnames(ev)[[3]] <- dimnames(pr)[[3]] <- colnames(model.response(object$model))
        class(ev) <- class(pr) <- c("ei", "array")    
        qi$ev <- ev
        qi$pr <- pr
        qi.name <- list(ev = "Expected In sample predictions at aggregate
    level", pr = "In sample predictions at aggregate level")
        
      }
    
    list(qi=qi, qi.name=qi.name)
  }
}

==> Zelig_2.4-4.tar.gz: Zelig/R/MCMC/qi.MCMCZelig.R <==

qi.MCMCZelig <- function(object, simpar=NULL, x, x1 = NULL, y = NULL, ...) {

  model <- object$zelig
  qi <- list()
  check <- FALSE

  if (model %in% c("MCMClogit", "MCMCprobit", "MCMCoprobit", "MCMCmnl")) 
    check <- TRUE
  
  if (model %in% c("MCMClogit","MCMCprobit", "MCMCregress",
                   "MCMCpoisson","MCMCtobit")) {

    if (model == "MCMClogit") {
      coef <- object$coefficients
      eta <- coef %*% t(x)
      pr <- ev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(pr) <- dimnames(ev) <- dimnames(eta)
      ev <- 1/(1+exp(-eta))
      for (i in 1:ncol(ev)) 
       pr[,i] <- as.character(rbinom(length(ev[,i]), 1, ev[,i])) 

      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")
    }
    else if (model == "MCMCprobit") {
      coef <- object$coefficients
      eta <- coef %*% t(x)
      pr <- ev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(pr) <- dimnames(ev) <- dimnames(eta)
      ev <- pnorm(eta)
      for (i in 1:ncol(ev)) 
        pr[,i] <- as.character(rbinom(length(ev[,i]), 1, ev[,i]))
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")
    }
    else if (model =="MCMCregress") {
      coef <- object$coefficients[,1:(ncol(object$coefficients)-1)]
      eta <- coef %*% t(x)
      ev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(ev) <- dimnames(eta)
      ev <- eta
      qi$ev <- ev
      qi.name <- list(ev = "Expected Values: E(Y|X)")
    }
    else if (model =="MCMCtobit") {
      coef <- object$coefficients[,1:(ncol(object$coefficients)-1)]
      sig2 <- object$coefficients[,ncol(object$coefficients)]
      sig <- sqrt(sig2)
      eta <- coef %*% t(x)
      ev <- cev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(cev) <- dimnames(ev) <- dimnames(eta)
      
      L2 <- (object$above-eta)/sig
      L1 <- (object$below-eta)/sig
      #cev <- eta + sig*(dnorm(L1)-dnorm(L2))/(pnorm(L2)-pnorm(L1))
      if (object$below==-Inf) temp1<-0
        else temp1 <- pnorm(L1)*object$below
      if (object$above==Inf) temp2<-0
        else temp2 <- (1-pnorm(L2))*object$above

      ev <- temp1+eta*(pnorm(L2)-pnorm(L1))+sig*(dnorm(L1)-dnorm(L2))+temp2

    qi$ev <- ev

    qi.name <- list(ev = "Expected Values: E(Y|X)")
    }
    else if (model == "MCMCpoisson") {
      coef <- object$coefficients
      eta <- coef %*% t(x)
      pr <- ev <- matrix(NA, nrow = nrow(eta), ncol = ncol(eta))
      dimnames(pr) <- dimnames(ev) <- dimnames(eta)
      ev <- exp(eta)
      for (i in 1:ncol(ev)) 
        pr[,i] <- rpois(length(ev[,i]), ev[,i])
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")

    }
    
        
    if (!is.null(x1)) {
        eta1 <- coef %*% t(x1)
      if (model == "MCMClogit") {
        ev1 <- 1/(1+exp(-eta1))
        rr <-ev1/ev
        fd <-ev1-ev

        qi$fd <- fd
        qi$rr <- rr

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

        qi.name$rr <- "Risk Ratios: P(Y=1|X1)/P(Y=1|X)"
      }
      else if (model == "MCMCprobit") {
        ev1 <- pnorm(eta1)
                rr <-ev1/ev
        fd <-ev1-ev

        qi$fd <- fd
        qi$rr <- rr

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

        qi.name$rr <- "Risk Ratios: P(Y=1|X1)/P(Y=1|X)"
      }
      else if (model == "MCMCregress") {
        ev1 <- eta1
        fd <-ev1-ev

        qi$fd <- fd

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"
      }
      else if (model == "MCMCtobit") {
        L2 <- (object$above-eta1)/sig
        L1 <- (object$below-eta1)/sig
        #cev <- eta + sig*(dnorm(L1)-dnorm(L2))/(pnorm(L2)-pnorm(L1))

        if (object$below==-Inf) temp1<-0
        else temp1 <- pnorm(L1)*object$below
        if (object$above==Inf) temp2<-0
        else temp2 <- (1-pnorm(L2))*object$above

        ev1 <- temp1+eta*(pnorm(L2)-pnorm(L1))+sig*(dnorm(L1)-dnorm(L2))+temp2
        
        fd <-ev1-ev

        qi$fd <- fd

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"
      }        
      else if (model == "MCMCpoisson") {
        ev1 <- exp(eta1)
        fd <-ev1-ev

        qi$fd <- fd

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

      }
   }

    if (!is.null(y)) {
    yvar <- matrix(rep(y, nrow(simpar)), nrow = nrow(simpar), byrow = TRUE)
    tmp.ev <- yvar - qi$ev
    if (check) 
      tmp.pr <- yvar - as.integer(qi$pr)
   else
      tmp.pr <- yvar - qi$pr
    qi$ate.ev <- matrix(apply(tmp.ev, 1, mean), nrow = nrow(simpar))
    qi.name$ate.ev <- "Average Treatment Effect: Y - EV"
    if (model %in% c("MCMClogit", "MCMCprobit", "MCMCpoisson"))
      {
        qi$ate.pr <- matrix(apply(tmp.pr, 1, mean), nrow = nrow(simpar))
        qi.name$ate.pr <- "Average Treatment Effect: Y - PR"
      }
  }

    list(qi=qi, qi.name=qi.name)    
  }
  else if ((model =="MCMCoprobit") || (model == "MCMCmnl")) {
    if (model == "MCMCoprobit") {
      library(stats)
      p <- dim(model.matrix(object, data=eval(object$data)))[2]
      coef <- object$coefficients[,1:p]
      eta <- coef %*% t(x)
      
      level <- ncol(object$coefficients)-p+2
      gamma<-matrix(NA, nrow(object$coefficients), level+1) 

      gamma[,1] <- rep(-Inf, nrow(gamma))
      gamma[,2] <- rep(0, nrow(gamma))
      gamma[,ncol(gamma)]<-rep(Inf, nrow(gamma))

      if (ncol(gamma)>3)
        gamma[,3:(ncol(gamma)-1)] <- object$coefficients[,(p+1):ncol(object$coefficients)]
 

      ev <- array(NA, c(nrow(eta), level, ncol(eta)))
      pr <- matrix(NA, nrow(eta), ncol(eta))
#      dimnames(pr)[1] <- dimnames(ev)[1] <- dimnames(eta)[1]
#      dimnames(pr)[2] <- dimnames(ev)[3] <- dimnames(eta)[2]

      for (j in 1:level)
        ev[,j,] <- pnorm(gamma[,j+1]-eta) - pnorm(gamma[,j]-eta)

      for (j in 1:nrow(pr)) {
       mu <- eta[j,]
       pr[j,]<-as.character(cut(mu, gamma[j,], labels=as.factor(1:level)))   
     }
       

      #        t(t(1:nrow(pr))%*%pr)
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")      
    }
    else if (model == "MCMCmnl") {
      library(stats)
      resp <- model.response(model.frame(object))
      level <- length(table(resp))

      p <- dim(model.matrix(eval(object),data=eval(object$data)))[2]
      
      coef <- object$coefficients


      eta <- array(NA, c(nrow(coef),level, nrow(x)))
      
      eta[,1,]<-matrix(0, dim(eta)[1],dim(eta)[3])
      for (j in 2:level) {
        ind <- (1:p)*(level-1)-(level-j)
        eta[,j,]<- coef[,ind]%*%t(x)
      }

      eta<-exp(eta)
                                      
      ev <- array(NA, c(nrow(coef), level, nrow(x)))
      pr <- matrix(NA, nrow(coef), nrow(x))
      #dimnames(pr)[1] <- dimnames(ev)[1] <- dimnames(eta)[1]

      for (k in 1:nrow(x))
       for (j in 1:level)
         {
           ev[,j,k] <- eta[,j,k]/rowSums(eta[,,k])
         }

      for (k in 1:nrow(x)) {             
        probs <- as.matrix(ev[,,k])
        temp <- apply(probs, 1, FUN=rmultinom, n=1, size=1)
        temp <- as.matrix(t(temp)%*%(1:nrow(temp)))
        pr <- apply(temp,2,as.character)
    }
      qi$ev <- ev
      qi$pr <- pr
      qi.name <- list(ev = "Expected Values: E(Y|X)", pr="Predicted
      Values: Y|X")      
    }

    
    if (!is.null(x1)) {

      if (model == "MCMCoprobit") {
 
        eta1 <- coef %*% t(x1)
      
      ev1 <- array(NA, c(nrow(eta), level, ncol(eta)))
      
      for (j in 1:level)
        ev1[,j,] <- pnorm(gamma[,j+1]-eta1) - pnorm(gamma[,j]-eta1)

        rr <-ev1/ev
        fd <-ev1-ev

        qi$fd <- fd
        qi$rr <- rr

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

        qi.name$rr <- "Risk Ratios: P(Y=1|X1)/P(Y=1|X)"
        
      }
      else if (model == "MCMCmnl") {

      eta1 <- array(NA, c(nrow(coef),level, nrow(x1)))
      
      eta1[,1,]<-matrix(0, dim(eta1)[1],dim(eta1)[3])
      for (j in 2:level) {
        ind <- (1:p)*(level-1)-(level-j)
        eta1[,j,]<- coef[,ind]%*%t(x1)
      }

      eta1<-exp(eta1)
                                       
      ev1 <- array(NA, c(nrow(eta1), level, nrow(x1)))

      for (k in 1:nrow(x))
       for (j in 1:level)
         {
           ev1[,j,k] <- eta1[,j,k]/rowSums(eta1[,,k])
         }

              rr <-ev1/ev
        fd <-ev1-ev

        qi$fd <- fd
        qi$rr <- rr

        qi.name$fd <- "First Differences in Expected Values: E(Y|X1)-E(Y|X)"

        qi.name$rr <- "Risk Ratios: P(Y=1|X1)/P(Y=1|X)"
      
    }
    }

    if (!is.null(y)) {
    yvar <- matrix(rep(y, nrow(simpar)), nrow = nrow(simpar), byrow =
    TRUE)

    levels.names<-levels(as.factor(y))
    yvar1<- pr1 <-array(NA, c(nrow(yvar), level, ncol(yvar)))


    for (j in 1:nrow(yvar))
      {
        yvar1[j,,]<-t(class.ind(yvar[j,], levels.names))
        if (check)
         pr1[j,,]<-t(class.ind(as.integer(qi$pr[j,]), levels.names))
        else
          pr1[j,,]<-t(class.ind(qi$pr[j,],levels.names))
      }
                        
    tmp.ev <- yvar1 - qi$ev
    tmp.pr <- yvar1 - pr1
    qi$ate.ev <- matrix(apply(tmp.ev, 2, rowMeans), nrow = nrow(simpar))
    qi.name$ate.ev <- "Average Treatment Effect: Y - EV"
    if (model %in% c("MCMCoprobit", "MCMCmnl"))
      {
        qi$ate.pr <- matrix(apply(tmp.pr, 2, rowMeans), nrow = nrow(simpar))
        qi.name$ate.pr <- "Average Treatment Effect: Y - PR"
      }
  }

    list(qi=qi, qi.name=qi.name)    


    
  }
  else if (model == "MCMChierEI" || model == "MCMCdynamicEI") {
    if (!any(class(x)=="cond")) stop("set 'cond=TRUE' in setx.\n")
    else
      {
        coef <- object$coefficients
        n <- nrow(x)
        if (is.null(object$N))
          N<-rep(1,n)
        else N <- eval(object$N)
        ev <- array(NA, c(nrow = nrow(coef), 2,2, n))
        pr <- array(NA, c(nrow = nrow(coef), 2,2, n))
        nlen<-length(coef[,1])
        for (j in 1:2) {
          ev[,j,1,] <- t(apply(coef[,((1:n)+(j-1)*n)],
                               1,"*",  x[,j])*N)
          ev[,j,2,] <- t(apply((1-coef[,((1:n)+(j-1)*n)]), 1,"*",
                               x[,j])*N)
          for (i in 1:n)
            {
              size<-round(x[i,j]*N[i])
              pr[,j,1,i] <-rbinom(prob=coef[,(i+(j-1)*n)],  n=nlen, size=size)

              pr[,j,2,i] <- x[i,j]*N[i]-pr[,j,1,i]
            }
          
        }
        
        dimnames(ev)[[4]] <- dimnames(pr)[[4]] <- rownames(x)
        dimnames(ev)[[2]] <- dimnames(pr)[[2]] <- colnames(x)
        dimnames(ev)[[3]] <- dimnames(pr)[[3]] <- colnames(model.response(object$model))
        class(ev) <- class(pr) <- c("ei", "array")    
        qi$ev <- ev
        qi$pr <- pr
        qi.name <- list(ev = "Expected In sample predictions at aggregate
    level", pr = "In sample predictions at aggregate level")
        
      }
    
    list(qi=qi, qi.name=qi.name)
  }
  else if ( model %in% c("MCMCfactanal", "MCMCordfactanal",
  "MCMCmixfactanal", "MCMCirt1d", "MCMCirtKd"))
    {
      stop("sim procedure not applicable since no explanatory
  variables are involved.\n")
    list(qi=qi)
    }

#  return(list(qi=qi, qi.name=qi.name))

}
